from __future__ import annotations

import json
from typing import AsyncGenerator

from aiohttp import ClientSession

from ..typing import AsyncResult, Messages
from .base_provider import AsyncGeneratorProvider, ProviderModelMixin
from .helper import format_prompt


class Snova(AsyncGeneratorProvider, ProviderModelMixin):
    url = "https://fast.snova.ai"
    api_endpoint = "https://fast.snova.ai/api/completion"
    working = True
    supports_stream = True
    supports_system_message = True
    supports_message_history = True
    
    default_model = 'Meta-Llama-3.1-8B-Instruct'
    models = [
        'Meta-Llama-3.1-8B-Instruct',
        'Meta-Llama-3.1-70B-Instruct',
        'Meta-Llama-3.1-405B-Instruct',
        'Samba-CoE',
        'ignos/Mistral-T5-7B-v1', # Error with the answer
        'v1olet/v1olet_merged_dpo_7B',
        'macadeliccc/WestLake-7B-v2-laser-truthy-dpo',
    ]
    
    model_aliases = {
        "llama-3.1-8b": "Meta-Llama-3.1-8B-Instruct",
        "llama-3.1-70b": "Meta-Llama-3.1-70B-Instruct",
        "llama-3.1-405b": "Meta-Llama-3.1-405B-Instruct",
        
        "mistral-7b": "ignos/Mistral-T5-7B-v1",
        
        "samba-coe-v0.1": "Samba-CoE",
        "v1olet-merged-7b": "v1olet/v1olet_merged_dpo_7B",
        "westlake-7b-v2": "macadeliccc/WestLake-7B-v2-laser-truthy-dpo",
    }

    @classmethod
    def get_model(cls, model: str) -> str:
        if model in cls.models:
            return model
        elif model in cls.model_aliases:
            return cls.model_aliases[model]
        else:
            return cls.default_model

    @classmethod
    async def create_async_generator(
        cls,
        model: str,
        messages: Messages,
        proxy: str = None,
        **kwargs
    ) -> AsyncGenerator[str, None]:
        model = cls.get_model(model)
        
        headers = {
            "accept": "text/event-stream",
            "accept-language": "en-US,en;q=0.9",
            "cache-control": "no-cache",
            "content-type": "application/json",
            "origin": cls.url,
            "pragma": "no-cache",
            "priority": "u=1, i",
            "referer": f"{cls.url}/",
            "sec-ch-ua": '"Chromium";v="127", "Not)A;Brand";v="99"',
            "sec-ch-ua-mobile": "?0",
            "sec-ch-ua-platform": '"Linux"',
            "sec-fetch-dest": "empty",
            "sec-fetch-mode": "cors",
            "sec-fetch-site": "same-origin",
            "user-agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/127.0.0.0 Safari/537.36"
        }
        async with ClientSession(headers=headers) as session:
            data = {
                "body": {
                    "messages": [
                        {
                            "role": "system",
                            "content": "You are a helpful assistant."
                        },
                        {
                            "role": "user",
                            "content": format_prompt(messages),
                            "id": "1-id",
                            "ref": "1-ref",
                            "revision": 1,
                            "draft": False,
                            "status": "done",
                            "enableRealTimeChat": False,
                            "meta": None
                        }
                    ],
                    "max_tokens": 1000,
                    "stop": ["<|eot_id|>"],
                    "stream": True,
                    "stream_options": {"include_usage": True},
                    "model": model
                },
                "env_type": "tp16"
            }
            async with session.post(cls.api_endpoint, json=data, proxy=proxy) as response:
                response.raise_for_status()
                full_response = ""
                async for line in response.content:
                    line = line.decode().strip()
                    if line.startswith("data: "):
                        data = line[6:]
                        if data == "[DONE]":
                            break
                        try:
                            json_data = json.loads(data)
                            choices = json_data.get("choices", [])
                            if choices:
                                delta = choices[0].get("delta", {})
                                content = delta.get("content", "")
                                full_response += content
                        except json.JSONDecodeError:
                            continue
                        except Exception as e:
                            print(f"Error processing chunk: {e}")
                            print(f"Problematic data: {data}")
                            continue
                
                yield full_response.strip()



from __future__ import annotations

from aiohttp import ClientSession
import json
import random
from .base_provider import AsyncGeneratorProvider, ProviderModelMixin
from ..typing import AsyncResult, Messages
from .helper import format_prompt

class Bixin123(AsyncGeneratorProvider, ProviderModelMixin):
    url = "https://chat.bixin123.com"
    api_endpoint = "https://chat.bixin123.com/api/chatgpt/chat-process"
    working = True
    supports_gpt_35_turbo = True
    supports_gpt_4 = True

    default_model = 'gpt-3.5-turbo-0125'
    models = ['gpt-3.5-turbo-0125', 'gpt-3.5-turbo-16k-0613', 'gpt-4-turbo', 'qwen-turbo']
    
    model_aliases = {
        "gpt-3.5-turbo": "gpt-3.5-turbo-0125",
        "gpt-3.5-turbo": "gpt-3.5-turbo-16k-0613",
    }

    @classmethod
    def get_model(cls, model: str) -> str:
        if model in cls.models:
            return model
        elif model in cls.model_aliases:
            return cls.model_aliases[model]
        else:
            return cls.default_model

    @classmethod
    def generate_fingerprint(cls) -> str:
        return str(random.randint(100000000, 999999999))

    @classmethod
    async def create_async_generator(
        cls,
        model: str,
        messages: Messages,
        proxy: str = None,
        **kwargs
    ) -> AsyncResult:
        model = cls.get_model(model)

        headers = {
            "accept": "application/json, text/plain, */*",
            "accept-language": "en-US,en;q=0.9",
            "cache-control": "no-cache",
            "content-type": "application/json",
            "fingerprint": cls.generate_fingerprint(),
            "origin": cls.url,
            "pragma": "no-cache",
            "priority": "u=1, i",
            "referer": f"{cls.url}/chat",
            "sec-ch-ua": '"Chromium";v="127", "Not)A;Brand";v="99"',
            "sec-ch-ua-mobile": "?0",
            "sec-ch-ua-platform": '"Linux"',
            "sec-fetch-dest": "empty",
            "sec-fetch-mode": "cors",
            "sec-fetch-site": "same-origin",
            "user-agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/127.0.0.0 Safari/537.36",
            "x-website-domain": "chat.bixin123.com",
        }

        async with ClientSession(headers=headers) as session:
            prompt = format_prompt(messages)
            data = {
                "prompt": prompt,
                "options": {
                    "usingNetwork": False,
                    "file": ""
                }
            }
            async with session.post(cls.api_endpoint, json=data, proxy=proxy) as response:
                response.raise_for_status()
                response_text = await response.text()
                
                lines = response_text.strip().split("\n")
                last_json = None
                for line in reversed(lines):
                    try:
                        last_json = json.loads(line)
                        break
                    except json.JSONDecodeError:
                        pass
                
                if last_json:
                    text = last_json.get("text", "")
                    yield text
                else:
                    yield ""



from __future__ import annotations

from aiohttp import ClientSession
from asyncio import sleep

from ..typing import AsyncResult, Messages
from .base_provider import AsyncGeneratorProvider, ProviderModelMixin
from .helper import format_prompt


class CodeNews(AsyncGeneratorProvider, ProviderModelMixin):
    url = "https://codenews.cc"
    api_endpoint = "https://codenews.cc/chatxyz13"
    working = True
    supports_gpt_35_turbo = True
    supports_gpt_4 = False
    supports_stream = True
    supports_system_message = False
    supports_message_history = False
    
    default_model = 'free_gpt'
    models = ['free_gpt', 'gpt-4o-mini', 'deepseek-coder', 'chatpdf']
    
    model_aliases = {
        "glm-4": "free_gpt",
        "gpt-3.5-turbo": "chatpdf",
        "deepseek": "deepseek-coder",
    }

    @classmethod
    def get_model(cls, model: str) -> str:
        if model in cls.models:
            return model
        elif model in cls.model_aliases:
            return cls.model_aliases[model]
        else:
            return cls.default_model

    @classmethod
    async def create_async_generator(
        cls,
        model: str,
        messages: Messages,
        proxy: str = None,
        **kwargs
    ) -> AsyncResult:
        model = cls.get_model(model)
        
        headers = {
            "accept": "application/json, text/javascript, */*; q=0.01",
            "accept-language": "en-US,en;q=0.9",
            "cache-control": "no-cache",
            "content-type": "application/x-www-form-urlencoded; charset=UTF-8",
            "origin": cls.url,
            "pragma": "no-cache",
            "priority": "u=1, i",
            "referer": f"{cls.url}/chatgpt",
            "sec-ch-ua": '"Chromium";v="127", "Not)A;Brand";v="99"',
            "sec-ch-ua-mobile": "?0",
            "sec-ch-ua-platform": '"Linux"',
            "sec-fetch-dest": "empty",
            "sec-fetch-mode": "cors",
            "sec-fetch-site": "same-origin",
            "user-agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/127.0.0.0 Safari/537.36",
            "x-requested-with": "XMLHttpRequest",
        }
        async with ClientSession(headers=headers) as session:
            prompt = format_prompt(messages)
            data = {
                "chatgpt_input": prompt,
                "qa_type2": model,
                "chatgpt_version_value": "20240804",
                "enable_web_search": "0",
                "enable_agent": "0",
                "dy_video_text_extract": "0",
                "enable_summary": "0",
            }
            async with session.post(cls.api_endpoint, data=data, proxy=proxy) as response:
                response.raise_for_status()
                json_data = await response.json()
                chat_id = json_data["data"]["id"]

            headers["content-type"] = "application/x-www-form-urlencoded; charset=UTF-8"
            data = {"current_req_count": "2"}
            
            while True:
                async with session.post(f"{cls.url}/chat_stream", headers=headers, data=data, proxy=proxy) as response:
                    response.raise_for_status()
                    json_data = await response.json()
                    if json_data["data"]:
                        yield json_data["data"]
                        break
                    else:
                        await sleep(1)  # Затримка перед наступним запитом



from __future__ import annotations

import os

from . import debug, version
from .models import Model
from .typing import Messages, CreateResult, AsyncResult, Union
from .errors import StreamNotSupportedError, ModelNotAllowedError
from .cookies import get_cookies, set_cookies
from .providers.types import ProviderType
from .providers.base_provider import AsyncGeneratorProvider
from .client.service import get_model_and_provider, get_last_provider

class ChatCompletion:
    @staticmethod
    def create(model    : Union[Model, str],
               messages : Messages,
               provider : Union[ProviderType, str, None] = None,
               stream   : bool = False,
               auth     : Union[str, None] = None,
               ignored  : list[str] = None, 
               ignore_working: bool = False,
               ignore_stream: bool = False,
               patch_provider: callable = None,
               **kwargs) -> Union[CreateResult, str]:
        """
        Creates a chat completion using the specified model, provider, and messages.

        Args:
            model (Union[Model, str]): The model to use, either as an object or a string identifier.
            messages (Messages): The messages for which the completion is to be created.
            provider (Union[ProviderType, str, None], optional): The provider to use, either as an object, a string identifier, or None.
            stream (bool, optional): Indicates if the operation should be performed as a stream.
            auth (Union[str, None], optional): Authentication token or credentials, if required.
            ignored (list[str], optional): List of provider names to be ignored.
            ignore_working (bool, optional): If True, ignores the working status of the provider.
            ignore_stream (bool, optional): If True, ignores the stream and authentication requirement checks.
            patch_provider (callable, optional): Function to modify the provider.
            **kwargs: Additional keyword arguments.

        Returns:
            Union[CreateResult, str]: The result of the chat completion operation.

        Raises:
            AuthenticationRequiredError: If authentication is required but not provided.
            ProviderNotFoundError, ModelNotFoundError: If the specified provider or model is not found.
            ProviderNotWorkingError: If the provider is not operational.
            StreamNotSupportedError: If streaming is requested but not supported by the provider.
        """
        model, provider = get_model_and_provider(
            model, provider, stream,
            ignored, ignore_working,
            ignore_stream or kwargs.get("ignore_stream_and_auth")
        )

        if auth is not None:
            kwargs['auth'] = auth
        
        if "proxy" not in kwargs:
            proxy = os.environ.get("G4F_PROXY")
            if proxy:
                kwargs['proxy'] = proxy

        if patch_provider:
            provider = patch_provider(provider)

        result = provider.create_completion(model, messages, stream, **kwargs)
        return result if stream else ''.join([str(chunk) for chunk in result])

    @staticmethod
    def create_async(model    : Union[Model, str],
                     messages : Messages,
                     provider : Union[ProviderType, str, None] = None,
                     stream   : bool = False,
                     ignored  : list[str] = None,
                     ignore_working: bool = False,
                     patch_provider: callable = None,
                     **kwargs) -> Union[AsyncResult, str]:
        """
        Asynchronously creates a completion using the specified model and provider.

        Args:
            model (Union[Model, str]): The model to use, either as an object or a string identifier.
            messages (Messages): Messages to be processed.
            provider (Union[ProviderType, str, None]): The provider to use, either as an object, a string identifier, or None.
            stream (bool): Indicates if the operation should be performed as a stream.
            ignored (list[str], optional): List of provider names to be ignored.
            patch_provider (callable, optional): Function to modify the provider.
            **kwargs: Additional keyword arguments.

        Returns:
            Union[AsyncResult, str]: The result of the asynchronous chat completion operation.

        Raises:
            StreamNotSupportedError: If streaming is requested but not supported by the provider.
        """
        model, provider = get_model_and_provider(model, provider, False, ignored, ignore_working)

        if stream:
            if isinstance(provider, type) and issubclass(provider, AsyncGeneratorProvider):
                return provider.create_async_generator(model, messages, **kwargs)
            raise StreamNotSupportedError(f'{provider.__name__} does not support "stream" argument in "create_async"')

        if patch_provider:
            provider = patch_provider(provider)

        return provider.create_async(model, messages, **kwargs)

class Completion:
    @staticmethod
    def create(model    : Union[Model, str],
               prompt   : str,
               provider : Union[ProviderType, None] = None,
               stream   : bool = False,
               ignored  : list[str] = None, **kwargs) -> Union[CreateResult, str]:
        """
        Creates a completion based on the provided model, prompt, and provider.

        Args:
            model (Union[Model, str]): The model to use, either as an object or a string identifier.
            prompt (str): The prompt text for which the completion is to be created.
            provider (Union[ProviderType, None], optional): The provider to use, either as an object or None.
            stream (bool, optional): Indicates if the operation should be performed as a stream.
            ignored (list[str], optional): List of provider names to be ignored.
            **kwargs: Additional keyword arguments.

        Returns:
            Union[CreateResult, str]: The result of the completion operation.

        Raises:
            ModelNotAllowedError: If the specified model is not allowed for use with this method.
        """
        allowed_models = [
            'code-davinci-002',
            'text-ada-001',
            'text-babbage-001',
            'text-curie-001',
            'text-davinci-002',
            'text-davinci-003'
        ]
        if model not in allowed_models:
            raise ModelNotAllowedError(f'Can\'t use {model} with Completion.create()')

        model, provider = get_model_and_provider(model, provider, stream, ignored)

        result = provider.create_completion(model, [{"role": "user", "content": prompt}], stream, **kwargs)

        return result if stream else ''.join(result)


from __future__ import annotations

import time
import random
import hashlib
import re
from aiohttp import ClientSession

from ..typing import AsyncResult, Messages
from .base_provider import AsyncGeneratorProvider, ProviderModelMixin
from .helper import format_prompt

class MagickPen(AsyncGeneratorProvider, ProviderModelMixin):
    url = "https://magickpen.com"
    api_endpoint_free = "https://api.magickpen.com/chat/free"
    api_endpoint_ask = "https://api.magickpen.com/ask"
    working = True
    supports_gpt_4 = True
    supports_stream = False
    
    default_model = 'free'
    models = ['free', 'ask']
    
    model_aliases = {
        "gpt-4o-mini": "free",
        "gpt-4o-mini": "ask",
    }

    @classmethod
    def get_model(cls, model: str) -> str:
        if model in cls.models:
            return model
        elif model in cls.model_aliases:
            return cls.model_aliases[model]
        else:
            return cls.default_model

    @classmethod
    async def get_secrets(cls):
        url = 'https://magickpen.com/_nuxt/02c76dc.js'
        async with ClientSession() as session:
            async with session.get(url) as response:
                if response.status == 200:
                    text = await response.text()
                    x_api_secret_match = re.search(r'"X-API-Secret":"([^"]+)"', text)
                    secret_match = re.search(r'secret:\s*"([^"]+)"', text)
                    
                    x_api_secret = x_api_secret_match.group(1) if x_api_secret_match else None
                    secret = secret_match.group(1) if secret_match else None
                    
                    # Generate timestamp and nonce dynamically
                    timestamp = str(int(time.time() * 1000))
                    nonce = str(random.random())
                    
                    # Generate signature
                    signature_parts = ["TGDBU9zCgM", timestamp, nonce]
                    signature_string = "".join(sorted(signature_parts))
                    signature = hashlib.md5(signature_string.encode()).hexdigest()
                    
                    return {
                        'X-API-Secret': x_api_secret,
                        'signature': signature,
                        'timestamp': timestamp,
                        'nonce': nonce,
                        'secret': secret
                    }
                else:
                    print(f"Error while fetching the file: {response.status}")
                    return None

    @classmethod
    async def create_async_generator(
        cls,
        model: str,
        messages: Messages,
        proxy: str = None,
        **kwargs
    ) -> AsyncResult:
        model = cls.get_model(model)
        
        secrets = await cls.get_secrets()
        if not secrets:
            raise Exception("Failed to obtain necessary secrets")

        headers = {
            "accept": "application/json, text/plain, */*",
            "accept-language": "en-US,en;q=0.9",
            "cache-control": "no-cache",
            "content-type": "application/json",
            "nonce": secrets['nonce'],
            "origin": "https://magickpen.com",
            "pragma": "no-cache",
            "priority": "u=1, i",
            "referer": "https://magickpen.com/",
            "sec-ch-ua": '"Chromium";v="127", "Not)A;Brand";v="99"',
            "sec-ch-ua-mobile": "?0",
            "sec-ch-ua-platform": '"Linux"',
            "sec-fetch-dest": "empty",
            "sec-fetch-mode": "cors",
            "sec-fetch-site": "same-site",
            "secret": secrets['secret'],
            "signature": secrets['signature'],
            "timestamp": secrets['timestamp'],
            "user-agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/127.0.0.0 Safari/537.36",
            "x-api-secret": secrets['X-API-Secret']
        }
        
        async with ClientSession(headers=headers) as session:
            if model == 'free':
                data = {
                    "history": [{"role": "user", "content": format_prompt(messages)}]
                }
                async with session.post(cls.api_endpoint_free, json=data, proxy=proxy) as response:
                    response.raise_for_status()
                    result = await response.text()
                    yield result
            
            elif model == 'ask':
                data = {
                    "query": format_prompt(messages),
                    "plan": "Pay as you go"
                }
                async with session.post(cls.api_endpoint_ask, json=data, proxy=proxy) as response:
                    response.raise_for_status()
                    async for chunk in response.content:
                        if chunk:
                            yield chunk.decode()
            
            else:
                raise ValueError(f"Unknown model: {model}")



from __future__  import annotations

from dataclasses import dataclass

from .Provider import IterListProvider, ProviderType
from .Provider import (
    AiChatOnline,
    Airforce,
    Allyfy,
    Bing,
    Binjie,
    Bixin123,
    Blackbox,
    ChatGot,
    Chatgpt4Online,
    ChatGpt,
    Chatgpt4o,
    ChatgptFree,
    CodeNews,
    DDG,
    DeepInfra,
    DeepInfraImage,
    Free2GPT,
    FreeChatgpt,
    FreeGpt,
    FreeNetfly,
    Gemini,
    GeminiPro,
    GigaChat,
    HuggingChat,
    HuggingFace,
    Koala,
    Liaobots,
    MagickPen,
    MetaAI,
    Nexra,
    OpenaiChat,
    PerplexityLabs,
    Pi,
    Pizzagpt,
    Reka,
    Replicate,
    ReplicateHome,
    Snova,
    TeachAnything,
    TwitterBio,
    Upstage,
    You,
)


@dataclass(unsafe_hash=True)
class Model:
    """
    Represents a machine learning model configuration.

    Attributes:
        name (str): Name of the model.
        base_provider (str): Default provider for the model.
        best_provider (ProviderType): The preferred provider for the model, typically with retry logic.
    """
    name: str
    base_provider: str
    best_provider: ProviderType = None

    @staticmethod
    def __all__() -> list[str]:
        """Returns a list of all model names."""
        return _all_models

default = Model(
    name          = "",
    base_provider = "",
    best_provider = IterListProvider([
        DDG,
        FreeChatgpt,
        HuggingChat,
        Pizzagpt,
        ChatgptFree,
        ReplicateHome,
        Upstage,
        Blackbox,
        Bixin123,
        Binjie,
        Free2GPT,
        MagickPen,
    ])
)

############
### Text ###
############

### OpenAI ###
# gpt-3
gpt_3 = Model(
    name          = 'gpt-3',
    base_provider = 'OpenAI',
    best_provider = IterListProvider([
        Nexra,
    ])
)

# gpt-3.5
gpt_35_turbo = Model(
    name          = 'gpt-3.5-turbo',
    base_provider = 'OpenAI',
    best_provider = IterListProvider([
        Allyfy, TwitterBio, Nexra, Bixin123, CodeNews, Airforce,
    ])
)

# gpt-4
gpt_4o = Model(
    name          = 'gpt-4o',
    base_provider = 'OpenAI',
    best_provider = IterListProvider([
        Liaobots, Chatgpt4o, Airforce, 
        OpenaiChat
    ])
)

gpt_4o_mini = Model(
    name          = 'gpt-4o-mini',
    base_provider = 'OpenAI',
    best_provider = IterListProvider([
        DDG, Liaobots, You, FreeNetfly, Pizzagpt, ChatgptFree, AiChatOnline, CodeNews, MagickPen, Airforce,  
        OpenaiChat, Koala, ChatGpt    
    ])
)

gpt_4_turbo = Model(
    name          = 'gpt-4-turbo',
    base_provider = 'OpenAI',
    best_provider = IterListProvider([
        Nexra, Bixin123, Liaobots, Airforce, Bing
    ])
)

gpt_4 = Model(
    name          = 'gpt-4',
    base_provider = 'OpenAI',
    best_provider = IterListProvider([
        Chatgpt4Online, Nexra, Binjie, Airforce, Bing,
        gpt_4_turbo.best_provider, gpt_4o.best_provider, gpt_4o_mini.best_provider
    ])
)

### GigaChat ###
gigachat = Model(
    name          = 'GigaChat:latest',
    base_provider = 'gigachat',
    best_provider = GigaChat
)


### Meta ###
meta = Model(
    name          = "meta-ai",
    base_provider = "Meta",
    best_provider = MetaAI
)

llama_2_13b = Model(
    name          = "llama-2-13b",
    base_provider = "Meta",
    best_provider = IterListProvider([Airforce])
)

llama_3_8b = Model(
    name          = "llama-3-8b",
    base_provider = "Meta",
    best_provider = IterListProvider([Airforce, DeepInfra, Replicate])
)

llama_3_70b = Model(
    name          = "llama-3-70b",
    base_provider = "Meta",
    best_provider = IterListProvider([ReplicateHome, Airforce, DeepInfra, Replicate])
)

llama_3_1_8b = Model(
    name          = "llama-3.1-8b",
    base_provider = "Meta",
    best_provider = IterListProvider([Blackbox, Airforce, PerplexityLabs])
)

llama_3_1_70b = Model(
    name          = "llama-3.1-70b",
    base_provider = "Meta",
    best_provider = IterListProvider([DDG, HuggingChat, FreeGpt, Blackbox, TeachAnything, Free2GPT, Airforce, HuggingFace, PerplexityLabs])
)

llama_3_1_405b = Model(
    name          = "llama-3.1-405b",
    base_provider = "Meta",
    best_provider = IterListProvider([Blackbox, Airforce])
)


### Mistral ###
mistral_7b = Model(
    name          = "mistral-7b",
    base_provider = "Mistral",
    best_provider = IterListProvider([HuggingChat, Airforce, HuggingFace, DeepInfra])
)

mixtral_8x7b = Model(
    name          = "mixtral-8x7b",
    base_provider = "Mistral",
    best_provider = IterListProvider([HuggingChat, DDG, ReplicateHome, TwitterBio, Airforce, DeepInfra, HuggingFace])
)

mixtral_8x22b = Model(
    name          = "mixtral-8x22b",
    base_provider = "Mistral",
    best_provider = IterListProvider([Airforce])
)


### NousResearch ###
mixtral_8x7b_dpo = Model(
    name          = "mixtral-8x7b-dpo",
    base_provider = "NousResearch",
    best_provider = IterListProvider([HuggingChat, Airforce, HuggingFace])
)

yi_34b = Model(
    name = 'yi-34b',
    base_provider = 'NousResearch',
    best_provider = IterListProvider([Airforce])
)


### Microsoft ###
phi_3_mini_4k = Model(
    name          = "phi-3-mini-4k",
    base_provider = "Microsoft",
    best_provider = IterListProvider([HuggingChat, HuggingFace])
)


### Google ###
# gemini
gemini_pro = Model(
    name          = 'gemini-pro',
    base_provider = 'Google',
    best_provider = IterListProvider([GeminiPro, ChatGot, Liaobots, Airforce])
)

gemini_flash = Model(
    name          = 'gemini-flash',
    base_provider = 'Google',
    best_provider = IterListProvider([Blackbox, Liaobots, Airforce])
)

gemini = Model(
    name          = 'gemini',
    base_provider = 'Google',
    best_provider = IterListProvider([
        Gemini, 
        gemini_flash.best_provider, gemini_pro.best_provider
   ])
)


# gemma
gemma_2b = Model(
    name          = 'gemma-2b',
    base_provider = 'Google',
    best_provider = IterListProvider([ReplicateHome, Airforce])
)

gemma_2b_9b = Model(
    name          = 'gemma-2b-9b',
    base_provider = 'Google',
    best_provider = IterListProvider([Airforce])
)

gemma_2b_27b = Model(
    name          = 'gemma-2b-27b',
    base_provider = 'Google',
    best_provider = IterListProvider([Airforce])
)

### Anthropic ###
claude_2 = Model(
    name          = 'claude-2',
    base_provider = 'Anthropic',
    best_provider = IterListProvider([You])
)

claude_2_0 = Model(
    name          = 'claude-2.0',
    base_provider = 'Anthropic',
    best_provider = IterListProvider([Liaobots])
)

claude_2_1 = Model(
    name          = 'claude-2.1',
    base_provider = 'Anthropic',
    best_provider = IterListProvider([Liaobots])
)

claude_3_opus = Model(
    name          = 'claude-3-opus',
    base_provider = 'Anthropic',
    best_provider = IterListProvider([Liaobots])
)

claude_3_sonnet = Model(
    name          = 'claude-3-sonnet',
    base_provider = 'Anthropic',
    best_provider = IterListProvider([Liaobots])
)

claude_3_5_sonnet = Model(
    name          = 'claude-3-5-sonnet',
    base_provider = 'Anthropic',
    best_provider = IterListProvider([Liaobots])
)

claude_3_haiku = Model(
    name          = 'claude-3-haiku',
    base_provider = 'Anthropic',
    best_provider = IterListProvider([DDG, Liaobots])
)


### Reka AI ###
reka_core = Model(
    name = 'reka-core',
    base_provider = 'Reka AI',
    best_provider = Reka
)


### Blackbox ###
blackbox = Model(
    name = 'blackbox',
    base_provider = 'Blackbox',
    best_provider = Blackbox
)


### Databricks ###
dbrx_instruct = Model(
    name = 'dbrx-instruct',
    base_provider = 'Databricks',
    best_provider = IterListProvider([Airforce, DeepInfra])
)


### CohereForAI ###
command_r_plus = Model(
    name = 'command-r-plus',
    base_provider = 'CohereForAI',
    best_provider = IterListProvider([HuggingChat])
)


### iFlytek ###
sparkdesk_v1_1 = Model(
    name = 'sparkdesk-v1.1',
    base_provider = 'iFlytek',
    best_provider = IterListProvider([FreeChatgpt, Airforce])
)

### Qwen ###
qwen_1_5_14b = Model(
    name = 'qwen-1.5-14b',
    base_provider = 'Qwen',
    best_provider = IterListProvider([FreeChatgpt])
)

qwen_1_5_72b = Model(
    name = 'qwen-1.5-72b',
    base_provider = 'Qwen',
    best_provider = IterListProvider([Airforce])
)

qwen_1_5_110b = Model(
    name = 'qwen-1.5-110b',
    base_provider = 'Qwen',
    best_provider = IterListProvider([Airforce])
)

qwen_2_72b = Model(
    name = 'qwen-2-72b',
    base_provider = 'Qwen',
    best_provider = IterListProvider([Airforce])
)

qwen_turbo = Model(
    name = 'qwen-turbo',
    base_provider = 'Qwen',
    best_provider = IterListProvider([Bixin123])
)


### Zhipu AI ###
glm_3_6b = Model(
    name = 'glm-3-6b',
    base_provider = 'Zhipu AI',
    best_provider = IterListProvider([FreeChatgpt])
)

glm_4_9b = Model(
    name = 'glm-4-9B',
    base_provider = 'Zhipu AI',
    best_provider = IterListProvider([FreeChatgpt])
)

glm_4 = Model(
    name = 'glm-4',
    base_provider = 'Zhipu AI',
    best_provider = IterListProvider([
        CodeNews, 
        glm_3_6b.best_provider, glm_4_9b.best_provider
    ])
)

### 01-ai ###
yi_1_5_9b = Model(
    name = 'yi-1.5-9b',
    base_provider = '01-ai',
    best_provider = IterListProvider([FreeChatgpt])
)

### Upstage ###
solar_1_mini = Model(
    name = 'solar-1-mini',
    base_provider = 'Upstage',
    best_provider = IterListProvider([Upstage])
)

solar_10_7b = Model(
    name = 'solar-10-7b',
    base_provider = 'Upstage',
    best_provider = Airforce
)


### Pi ###
pi = Model(
    name = 'pi',
    base_provider = 'inflection',
    best_provider = Pi
)

### SambaNova ###
samba_coe_v0_1 = Model(
    name = 'samba-coe-v0.1',
    base_provider = 'SambaNova',
    best_provider = Snova
)

### Trong-Hieu Nguyen-Mau ###
v1olet_merged_7b = Model(
    name = 'v1olet-merged-7b',
    base_provider = 'Trong-Hieu Nguyen-Mau',
    best_provider = Snova
)

### Macadeliccc ###
westlake_7b_v2 = Model(
    name = 'westlake-7b-v2',
    base_provider = 'Macadeliccc',
    best_provider = Snova
)

### DeepSeek ###
deepseek = Model(
    name = 'deepseek',
    base_provider = 'DeepSeek',
    best_provider = IterListProvider([CodeNews, Airforce])
)

### WizardLM ###
wizardlm_2_8x22b = Model(
    name = 'wizardlm-2-8x22b',
    base_provider = 'WizardLM',
    best_provider = Airforce
)

### Together ###
sh_n_7b = Model(
    name = 'sh-n-7b',
    base_provider = 'Together',
    best_provider = Airforce
)

### Yorickvp ###
llava_13b = Model(
    name = 'llava-13b',
    base_provider = 'Yorickvp',
    best_provider = ReplicateHome
)

#############
### Image ###
#############

### Stability AI ###
sdxl = Model(
    name = 'sdxl',
    base_provider = 'Stability AI',
    best_provider = IterListProvider([ReplicateHome, DeepInfraImage])
    
)

sd_3 = Model(
    name = 'sd-3',
    base_provider = 'Stability AI',
    best_provider = IterListProvider([ReplicateHome])
    
)

### Playground ###
playground_v2_5 = Model(
    name = 'playground-v2.5',
    base_provider = 'Stability AI',
    best_provider = IterListProvider([ReplicateHome])
    
)

### Flux AI ###
flux = Model(
    name = 'flux',
    base_provider = 'Flux AI',
    best_provider = IterListProvider([Airforce])
    
)

flux_realism = Model(
    name = 'flux-realism',
    base_provider = 'Flux AI',
    best_provider = IterListProvider([Airforce])
    
)

flux_anime = Model(
    name = 'flux-anime',
    base_provider = 'Flux AI',
    best_provider = IterListProvider([Airforce])
    
)

flux_3d = Model(
    name = 'flux-3d',
    base_provider = 'Flux AI',
    best_provider = IterListProvider([Airforce])
    
)

flux_disney = Model(
    name = 'flux-disney',
    base_provider = 'Flux AI',
    best_provider = IterListProvider([Airforce])
    
)

flux_pixel = Model(
    name = 'flux-pixel',
    base_provider = 'Flux AI',
    best_provider = IterListProvider([Airforce])
    
)

flux_schnell = Model(
    name = 'flux-schnell',
    base_provider = 'Flux AI',
    best_provider = IterListProvider([ReplicateHome])
    
)

### ###
dalle = Model(
    name = 'dalle',
    base_provider = '',
    best_provider = IterListProvider([Nexra])
    
)

dalle_2 = Model(
    name = 'dalle-2',
    base_provider = '',
    best_provider = IterListProvider([Nexra])
    
)

dalle_mini = Model(
    name = 'dalle-mini',
    base_provider = '',
    best_provider = IterListProvider([Nexra])
    
)

emi = Model(
    name = 'emi',
    base_provider = '',
    best_provider = IterListProvider([Nexra])
    
)

any_dark = Model(
    name = 'any-dark',
    base_provider = '',
    best_provider = IterListProvider([Airforce])
    
)

class ModelUtils:
    """
    Utility class for mapping string identifiers to Model instances.

    Attributes:
        convert (dict[str, Model]): Dictionary mapping model string identifiers to Model instances.
    """
    convert: dict[str, Model] = {
    
############
### Text ###
############
        
### OpenAI ###
# gpt-3
'gpt-3': gpt_3,

# gpt-3.5
'gpt-3.5-turbo': gpt_35_turbo,

# gpt-4
'gpt-4o'         : gpt_4o,
'gpt-4o-mini'    : gpt_4o_mini,
'gpt-4'          : gpt_4,
'gpt-4-turbo'    : gpt_4_turbo,
        
        
### Meta ###
"meta-ai": meta,

# llama-2
'llama-2-13b': llama_2_13b,

# llama-3
'llama-3-8b': llama_3_8b,
'llama-3-70b': llama_3_70b,
        
# llama-3.1
'llama-3.1-8b': llama_3_1_8b,
'llama-3.1-70b': llama_3_1_70b,
'llama-3.1-405b': llama_3_1_405b,
      
        
### Mistral ###
'mistral-7b': mistral_7b,
'mixtral-8x7b': mixtral_8x7b,
'mixtral-8x22b': mixtral_8x22b,
     
     
### NousResearch ###
'mixtral-8x7b-dpo': mixtral_8x7b_dpo,  
 
'yi-34b': yi_34b,   
        
        
### Microsoft ###
'phi-3-mini-4k': phi_3_mini_4k,


### Google ###
# gemini
'gemini': gemini,
'gemini-pro': gemini_pro,
'gemini-flash': gemini_flash,
        
# gemma
'gemma-2b': gemma_2b,
'gemma-2b-9b': gemma_2b_9b,
'gemma-2b-27b': gemma_2b_27b,


### Anthropic ###
'claude-2': claude_2,
'claude-2.0': claude_2_0,
'claude-2.1': claude_2_1,
        
'claude-3-opus': claude_3_opus,
'claude-3-sonnet': claude_3_sonnet,
'claude-3-haiku': claude_3_haiku,
'claude-3-5-sonnet': claude_3_5_sonnet,
        
        
### Reka AI ###
'reka-core': reka_core,
      
        
### Blackbox ###
'blackbox': blackbox,
        
        
### CohereForAI ###
'command-r+': command_r_plus,
        
        
### Databricks ###
'dbrx-instruct': dbrx_instruct,


### GigaChat ###
'gigachat': gigachat,
        
        
### iFlytek ###
'sparkdesk-v1.1': sparkdesk_v1_1,
        
        
### Qwen ###
'qwen-1.5-14b': qwen_1_5_14b,
'qwen-1.5-72b': qwen_1_5_72b,
'qwen-1.5-110b': qwen_1_5_110b,
'qwen-2-72b': qwen_2_72b,
'qwen-turbo': qwen_turbo,
        
        
### Zhipu AI ###
'glm-3-6b': glm_3_6b,
'glm-4-9b': glm_4_9b,
'glm-4': glm_4,
        
        
### 01-ai ###
'yi-1.5-9b': yi_1_5_9b,
        
        
### Upstage ###
'solar-1-mini': solar_1_mini,
'solar-10-7b': solar_10_7b,


### Pi ###
'pi': pi,


### SambaNova ###
'samba-coe-v0.1': samba_coe_v0_1,


### Trong-Hieu Nguyen-Mau ###
'v1olet-merged-7b': v1olet_merged_7b,


### Macadeliccc ###
'westlake-7b-v2': westlake_7b_v2,


### DeepSeek ###
'deepseek': deepseek,


### Together ###
'sh-n-7b': sh_n_7b,
      
        
### Yorickvp ###
'llava-13b': llava_13b,
        
        
        
#############
### Image ###
#############
        
### Stability AI ###
'sdxl': sdxl,
'sd-3': sd_3,
        
        
### Playground ###
'playground-v2.5': playground_v2_5,


### Flux AI ###
'flux': flux,
'flux-realism': flux_realism,
'flux-anime': flux_anime,
'flux-3d': flux_3d,
'flux-disney': flux_disney,
'flux-pixel': flux_pixel,
'flux-schnell': flux_schnell,


###  ###
'dalle': dalle,
'dalle-2': dalle_2,
'dalle-mini': dalle_mini,
'emi': emi,
'any-dark': any_dark,
    }

_all_models = list(ModelUtils.convert.keys())



from __future__ import annotations

from ..typing import Messages, CreateResult
from ..providers.base_provider import AbstractProvider, ProviderModelMixin

import requests, os, re, json

class ChatGptEs(AbstractProvider, ProviderModelMixin):
    label = "ChatGptEs"
    working = True
    supports_message_history = True
    supports_system_message = True
    supports_stream = False

    @classmethod
    def create_completion(
        cls,
        model: str,
        messages: Messages,
        stream: bool,
        **kwargs
    ) -> CreateResult:
        
        if model not in [
            'gpt-4o',
            'gpt-4o-mini',
            'chatgpt-4o-latest'
        ]:
            raise ValueError(f"Unsupported model: {model}")

        headers = {
            'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7',
            'accept-language': 'en,fr-FR;q=0.9,fr;q=0.8,es-ES;q=0.7,es;q=0.6,en-US;q=0.5,am;q=0.4,de;q=0.3',
            'cache-control': 'no-cache',
            'pragma': 'no-cache',
            'priority': 'u=0, i',
            'referer': 'https://www.google.com/',
            'sec-ch-ua': '"Google Chrome";v="129", "Not=A?Brand";v="8", "Chromium";v="129"',
            'sec-ch-ua-mobile': '?0',
            'sec-ch-ua-platform': '"macOS"',
            'sec-fetch-dest': 'document',
            'sec-fetch-mode': 'navigate',
            'sec-fetch-site': 'cross-site',
            'sec-fetch-user': '?1',
            'upgrade-insecure-requests': '1',
            'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/129.0.0.0 Safari/537.36',
        }

        response = requests.get('https://chatgpt.es/', headers=headers)
        nonce_   = re.findall(r'data-nonce="(.+?)"', response.text)[0]
        post_id  = re.findall(r'data-post-id="(.+?)"', response.text)[0]

        headers = {
            'accept': '*/*',
            'accept-language': 'en,fr-FR;q=0.9,fr;q=0.8,es-ES;q=0.7,es;q=0.6,en-US;q=0.5,am;q=0.4,de;q=0.3',
            'cache-control': 'no-cache',
            'origin': 'https://chatgpt.es',
            'pragma': 'no-cache',
            'priority': 'u=1, i',
            'referer': 'https://chatgpt.es/',
            'sec-ch-ua': '"Google Chrome";v="129", "Not=A?Brand";v="8", "Chromium";v="129"',
            'sec-ch-ua-mobile': '?0',
            'sec-ch-ua-platform': '"macOS"',
            'sec-fetch-dest': 'empty',
            'sec-fetch-mode': 'cors',
            'sec-fetch-site': 'same-origin',
            'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/129.0.0.0 Safari/537.36',
        }
        
        conversation_history = [
            "Human: stricly respond in the same language as my prompt, preferably english"
        ]
        
        for message in messages[:-1]:
            if message['role'] == "user":
                conversation_history.append(f"Human: {message['content']}")
            else:
                conversation_history.append(f"AI: {message['content']}")

        payload = {
            '_wpnonce': nonce_,
            'post_id': post_id,
            'url': 'https://chatgpt.es',
            'action': 'wpaicg_chat_shortcode_message',
            'message': messages[-1]['content'],
            'bot_id': '0',
            'chatbot_identity': 'shortcode',
            'wpaicg_chat_client_id': os.urandom(5).hex(),
            'wpaicg_chat_history': json.dumps(conversation_history)
        }

        response = requests.post('https://chatgpt.es/wp-admin/admin-ajax.php', 
                                headers=headers, data=payload).json()
        
        return (response['data'])


from __future__ import annotations

from aiohttp import ClientSession, ClientResponseError
from urllib.parse import urlencode
import json
import io
import asyncio

from ..typing import AsyncResult, Messages
from .base_provider import AsyncGeneratorProvider, ProviderModelMixin
from ..image import ImageResponse, is_accepted_format
from .helper import format_prompt

class Airforce(AsyncGeneratorProvider, ProviderModelMixin):
    url = "https://api.airforce"
    text_api_endpoint = "https://api.airforce/chat/completions"
    image_api_endpoint = "https://api.airforce/v1/imagine2"
    working = True
    supports_gpt_35_turbo = True
    supports_gpt_4 = True
    supports_stream = True
    supports_system_message = True
    supports_message_history = True
    default_model = 'llama-3-70b-chat'
    text_models = [
        # Open source models
        'llama-2-13b-chat',
        
        'llama-3-70b-chat',
        'llama-3-70b-chat-turbo',
        'llama-3-70b-chat-lite',
        
        'llama-3-8b-chat',
        'llama-3-8b-chat-turbo',
        'llama-3-8b-chat-lite',
        
        'llama-3.1-405b-turbo',
        'llama-3.1-70b-turbo',
        'llama-3.1-8b-turbo',
        
        'LlamaGuard-2-8b',
        'Llama-Guard-7b',
        'Meta-Llama-Guard-3-8B',

        'Mixtral-8x7B-Instruct-v0.1',
        'Mixtral-8x22B-Instruct-v0.1',
        'Mistral-7B-Instruct-v0.1',
        'Mistral-7B-Instruct-v0.2',
        'Mistral-7B-Instruct-v0.3',
        
        'Qwen1.5-72B-Chat',
        'Qwen1.5-110B-Chat',
        'Qwen2-72B-Instruct',
        
        'gemma-2b-it',
        'gemma-2-9b-it',
        'gemma-2-27b-it',
        
        'dbrx-instruct',

        'deepseek-llm-67b-chat',
        
        'Nous-Hermes-2-Mixtral-8x7B-DPO',
        'Nous-Hermes-2-Yi-34B',
        
        'WizardLM-2-8x22B',
        
        'SOLAR-10.7B-Instruct-v1.0',
        
        'StripedHyena-Nous-7B',      
        
        'sparkdesk',
        
        
        # Other models
        'chatgpt-4o-latest',
        'gpt-4',
        'gpt-4-turbo',
        'gpt-4o-mini-2024-07-18',
        'gpt-4o-mini',
        'gpt-4o',
        'gpt-3.5-turbo',
        'gpt-3.5-turbo-0125',
        'gpt-3.5-turbo-1106',
        'gpt-3.5-turbo-16k',
        'gpt-3.5-turbo-0613',
        'gpt-3.5-turbo-16k-0613',
        
        'gemini-1.5-flash',
        'gemini-1.5-pro',
    ]
    image_models = [
        'flux',
        'flux-realism',
        'flux-anime',
        'flux-3d',
        'flux-disney',
        'flux-pixel',
        'any-dark',
    ]
    
    models = [
        *text_models,
        *image_models
    ]
    model_aliases = {
        # Open source models
        "llama-2-13b": "llama-2-13b-chat",
        
        "llama-3-70b": "llama-3-70b-chat",
        "llama-3-70b": "llama-3-70b-chat-turbo",
        "llama-3-70b": "llama-3-70b-chat-lite",
        
        "llama-3-8b": "llama-3-8b-chat",
        "llama-3-8b": "llama-3-8b-chat-turbo",
        "llama-3-8b": "llama-3-8b-chat-lite",
        
        "llama-3.1-405b": "llama-3.1-405b-turbo",
        "llama-3.1-70b": "llama-3.1-70b-turbo",
        "llama-3.1-8b": "llama-3.1-8b-turbo",
        
        "mixtral-8x7b": "Mixtral-8x7B-Instruct-v0.1",
        "mixtral-8x22b": "Mixtral-8x22B-Instruct-v0.1",
        "mistral-7b": "Mistral-7B-Instruct-v0.1",
        "mistral-7b": "Mistral-7B-Instruct-v0.2",
        "mistral-7b": "Mistral-7B-Instruct-v0.3",
        
        "mixtral-8x7b-dpo": "Nous-Hermes-2-Mixtral-8x7B-DPO",
        
        "qwen-1-5-72b": "Qwen1.5-72B-Chat",
        "qwen-1_5-110b": "Qwen1.5-110B-Chat",
        "qwen-2-72b": "Qwen2-72B-Instruct",
        
        "gemma-2b": "gemma-2b-it",
        "gemma-2b-9b": "gemma-2-9b-it",
        "gemma-2b-27b": "gemma-2-27b-it",
        
        "deepseek": "deepseek-llm-67b-chat",
        
        "yi-34b": "Nous-Hermes-2-Yi-34B",
        
        "wizardlm-2-8x22b": "WizardLM-2-8x22B",
        
        "solar-10-7b": "SOLAR-10.7B-Instruct-v1.0",
        
        "sh-n-7b": "StripedHyena-Nous-7B",
        
        "sparkdesk-v1.1": "sparkdesk",
        
        
        # Other models
        "gpt-4o": "chatgpt-4o-latest",
        "gpt-4o-mini": "gpt-4o-mini-2024-07-18",
        
        "gpt-3.5-turbo": "gpt-3.5-turbo-0125",
        "gpt-3.5-turbo": "gpt-3.5-turbo-1106",
        "gpt-3.5-turbo": "gpt-3.5-turbo-16k",
        "gpt-3.5-turbo": "gpt-3.5-turbo-0613",
        "gpt-3.5-turbo": "gpt-3.5-turbo-16k-0613",
        
        
        "gemini-flash": "gemini-1.5-flash",
        "gemini-pro": "gemini-1.5-pro",
    }

    @classmethod
    async def create_async_generator(
        cls,
        model: str,
        messages: Messages,
        proxy: str = None,
        **kwargs
    ) -> AsyncResult:
        model = cls.get_model(model)
        
        headers = {
            "accept": "*/*",
            "accept-language": "en-US,en;q=0.9",
            "content-type": "application/json",
            "origin": "https://api.airforce",
            "sec-ch-ua": '"Chromium";v="128", "Not(A:Brand";v="24"',
            "sec-ch-ua-mobile": "?0",
            "sec-ch-ua-platform": '"Linux"',
            "sec-fetch-dest": "empty",
            "sec-fetch-mode": "cors",
            "sec-fetch-site": "cross-site",
            "user-agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/128.0.0.0 Safari/537.36"
        }

        
        if model in cls.image_models:
            async for item in cls.generate_image(model, messages, headers, proxy, **kwargs):
                yield item
        else:
            async for item in cls.generate_text(model, messages, headers, proxy, **kwargs):
                yield item

    @classmethod
    async def generate_text(cls, model: str, messages: Messages, headers: dict, proxy: str, **kwargs) -> AsyncResult:
        async with ClientSession(headers=headers) as session:
            data = {
                "messages": [{"role": "user", "content": format_prompt(messages)}],
                "model": model,
                "temperature": kwargs.get('temperature', 1),
                "top_p": kwargs.get('top_p', 1),
                "stream": True
            }
            
            async with session.post(cls.text_api_endpoint, json=data, proxy=proxy) as response:
                response.raise_for_status()
                async for line in response.content:
                    if line:
                        line = line.decode('utf-8').strip()
                        if line.startswith("data: "):
                            try:
                                data = json.loads(line[6:])
                                if 'choices' in data and len(data['choices']) > 0:
                                    delta = data['choices'][0].get('delta', {})
                                    if 'content' in delta:
                                        yield delta['content']
                            except json.JSONDecodeError:
                                continue
                        elif line == "data: [DONE]":
                            break

    @classmethod
    async def generate_image(cls, model: str, messages: Messages, headers: dict, proxy: str, **kwargs) -> AsyncResult:
        prompt = messages[-1]['content'] if messages else ""
        params = {
            "prompt": prompt,
            "size": kwargs.get("size", "1:1"),
            "seed": kwargs.get("seed"),
            "model": model
        }
        params = {k: v for k, v in params.items() if v is not None}

        try:
            async with ClientSession(headers=headers) as session:
                async with session.get(cls.image_api_endpoint, params=params, proxy=proxy) as response:
                    response.raise_for_status()
                    content = await response.read()
                    
                    if response.content_type.startswith('image/'):
                        image_url = str(response.url)
                        yield ImageResponse(image_url, prompt)
                    else:
                        try:
                            text = content.decode('utf-8', errors='ignore')
                            yield f"Error: {text}"
                        except Exception as decode_error:
                            yield f"Error: Unable to decode response - {str(decode_error)}"
        except ClientResponseError as e:
            yield f"Error: HTTP {e.status}: {e.message}"
        except Exception as e:
            yield f"Unexpected error: {str(e)}"



from __future__ import annotations

import re
import json
import random
import string
from aiohttp import ClientSession

from ..typing import AsyncResult, Messages, ImageType
from ..image import ImageResponse, to_data_uri
from .base_provider import AsyncGeneratorProvider, ProviderModelMixin

class Blackbox(AsyncGeneratorProvider, ProviderModelMixin):
    url = "https://www.blackbox.ai"
    api_endpoint = "https://www.blackbox.ai/api/chat"
    working = True
    supports_stream = True
    supports_system_message = True
    supports_message_history = True
    
    default_model = 'blackbox'
    models = [
        'blackbox',
        'gemini-1.5-flash',
        "llama-3.1-8b",
        'llama-3.1-70b',
        'llama-3.1-405b',
        'ImageGenerationLV45LJp'
    ]

    model_config = {
        "blackbox": {},
        "gemini-1.5-flash": {'mode': True, 'id': 'Gemini'},
        "llama-3.1-8b": {'mode': True, 'id': "llama-3.1-8b"},
        'llama-3.1-70b': {'mode': True, 'id': "llama-3.1-70b"},
        'llama-3.1-405b': {'mode': True, 'id': "llama-3.1-405b"},
        'ImageGenerationLV45LJp': {'mode': True, 'id': "ImageGenerationLV45LJp", 'name': "Image Generation"},
    }

    @classmethod
    def get_model(cls, model: str) -> str:
        if model in cls.models:
            return model
        elif model in cls.model_aliases:
            return cls.model_aliases[model]
        else:
            return cls.default_model

    @classmethod
    async def create_async_generator(
        cls,
        model: str,
        messages: Messages,
        proxy: str = None,
        image: ImageType = None,
        image_name: str = None,
        **kwargs
    ) -> AsyncResult:
        model = cls.get_model(model)
        
        headers = {
            "accept": "*/*",
            "accept-language": "en-US,en;q=0.9",
            "cache-control": "no-cache",
            "content-type": "application/json",
            "origin": cls.url,
            "pragma": "no-cache",
            "referer": f"{cls.url}/",
            "sec-ch-ua": '"Not;A=Brand";v="24", "Chromium";v="128"',
            "sec-ch-ua-mobile": "?0",
            "sec-ch-ua-platform": '"Linux"',
            "sec-fetch-dest": "empty",
            "sec-fetch-mode": "cors",
            "sec-fetch-site": "same-origin",
            "user-agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/128.0.0.0 Safari/537.36"
        }
        
        async with ClientSession(headers=headers) as session:
            if image is not None:
                messages[-1]["data"] = {
                    "fileText": image_name,
                    "imageBase64": to_data_uri(image)
                }
            
            random_id = ''.join(random.choices(string.ascii_letters + string.digits, k=7))

            data = {
                "messages": messages,
                "id": random_id,
                "previewToken": None,
                "userId": None,
                "codeModelMode": True,
                "agentMode": {},
                "trendingAgentMode": {},
                "isMicMode": False,
                "maxTokens": None,
                "isChromeExt": False,
                "githubToken": None,
                "clickedAnswer2": False,
                "clickedAnswer3": False,
                "clickedForceWebSearch": False,
                "visitFromDelta": False,
                "mobileClient": False
            }

            if model == 'ImageGenerationLV45LJp':
                data["agentMode"] = cls.model_config[model]
            else:
                data["trendingAgentMode"] = cls.model_config[model]
            
            async with session.post(cls.api_endpoint, json=data, proxy=proxy) as response:
                response.raise_for_status()
                if model == 'ImageGenerationLV45LJp':
                    response_text = await response.text()
                    url_match = re.search(r'https://storage\.googleapis\.com/[^\s\)]+', response_text)
                    if url_match:
                        image_url = url_match.group(0)
                        yield ImageResponse(image_url, alt=messages[-1]['content'])
                    else:
                        raise Exception("Image URL not found in the response")
                else:
                    async for chunk in response.content:
                        if chunk:
                            decoded_chunk = chunk.decode()
                            if decoded_chunk.startswith('$@$v=undefined-rv1$@$'):
                                decoded_chunk = decoded_chunk[len('$@$v=undefined-rv1$@$'):]
                            yield decoded_chunk



from __future__ import annotations

from ..typing import Messages, CreateResult
from ..providers.base_provider import AbstractProvider, ProviderModelMixin

import time, uuid, random, json
from requests import Session

from .openai.new import (
    get_config,
    get_answer_token,
    process_turnstile,
    get_requirements_token
)

def format_conversation(messages: list):
    conversation = []
    
    for message in messages:
        conversation.append({
            'id': str(uuid.uuid4()),
            'author': {
                'role': message['role'],
            },
            'content': {
                'content_type': 'text',
                'parts': [
                    message['content'],
                ],
            },
            'metadata': {
                'serialization_metadata': {
                    'custom_symbol_offsets': [],
                },
            },
            'create_time': round(time.time(), 3),
        })
    
    return conversation

def init_session(user_agent):
    session = Session()

    cookies = {
        '_dd_s': '',
    }

    headers = {
        'accept': '*/*',
        'accept-language': 'en-US,en;q=0.8',
        'cache-control': 'no-cache',
        'pragma': 'no-cache',
        'priority': 'u=0, i',
        'sec-ch-ua': '"Not)A;Brand";v="99", "Google Chrome";v="127", "Chromium";v="127"',
        'sec-ch-ua-arch': '"arm"',
        'sec-ch-ua-bitness': '"64"',
        'sec-ch-ua-mobile': '?0',
        'sec-ch-ua-model': '""',
        'sec-ch-ua-platform': '"macOS"',
        'sec-ch-ua-platform-version': '"14.4.0"',
        'sec-fetch-dest': 'document',
        'sec-fetch-mode': 'navigate',
        'sec-fetch-site': 'none',
        'sec-fetch-user': '?1',
        'upgrade-insecure-requests': '1',
        'user-agent': user_agent,
    }

    session.get('https://chatgpt.com/', cookies=cookies, headers=headers)
    
    return session

class ChatGpt(AbstractProvider, ProviderModelMixin):
    label = "ChatGpt"
    working = True
    supports_message_history = True
    supports_system_message = True
    supports_stream = True


    @classmethod
    def create_completion(
        cls,
        model: str,
        messages: Messages,
        stream: bool,
        **kwargs
    ) -> CreateResult:
        
        if model in [
            'gpt-4o',
            'gpt-4o-mini',
            'gpt-4',
            'gpt-4-turbo',
            'chatgpt-4o-latest'
        ]:
            model = 'auto'
        
        elif model in [
            'gpt-3.5-turbo'
        ]:
            model = 'text-davinci-002-render-sha'
            
        else:
            raise ValueError(f"Invalid model: {model}")
        
        user_agent       = 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/127.0.0.0 Safari/537.36'
        session: Session = init_session(user_agent)
        
        config   = get_config(user_agent)
        pow_req  = get_requirements_token(config)
        headers  = { 
            'accept': '*/*',
            'accept-language': 'en-US,en;q=0.8',
            'content-type': 'application/json',
            'oai-device-id': f'{uuid.uuid4()}',
            'oai-language': 'en-US',
            'origin': 'https://chatgpt.com',
            'priority': 'u=1, i',
            'referer': 'https://chatgpt.com/',
            'sec-ch-ua-mobile': '?0',
            'sec-ch-ua-platform': '"Linux"',
            'sec-fetch-dest': 'empty',
            'sec-fetch-mode': 'cors',
            'sec-fetch-site': 'same-origin',
            'sec-gpc': '1',
            'user-agent': f'{user_agent}'
        }
        
        response = session.post('https://chatgpt.com/backend-anon/sentinel/chat-requirements',
            headers=headers, json={'p': pow_req}).json()

        turnstile          = response.get('turnstile', {})
        turnstile_required = turnstile.get('required')
        pow_conf           = response.get('proofofwork', {})

        if turnstile_required:
            turnstile_dx    = turnstile.get('dx')
            turnstile_token = process_turnstile(turnstile_dx, pow_req)
        
        headers = headers | {
            'openai-sentinel-turnstile-token'        : turnstile_token,
            'openai-sentinel-chat-requirements-token': response.get('token'),
            'openai-sentinel-proof-token'            : get_answer_token(
                pow_conf.get('seed'), pow_conf.get('difficulty'), config
            )
        }
        
        json_data = {
            'action': 'next',
            'messages': format_conversation(messages),
            'parent_message_id': str(uuid.uuid4()),
            'model': 'auto',
            'timezone_offset_min': -120,
            'suggestions': [
                'Can you help me create a personalized morning routine that would help increase my productivity throughout the day? Start by asking me about my current habits and what activities energize me in the morning.',
                'Could you help me plan a relaxing day that focuses on activities for rejuvenation? To start, can you ask me what my favorite forms of relaxation are?',
                'I have a photoshoot tomorrow. Can you recommend me some colors and outfit options that will look good on camera?',
                'Make up a 5-sentence story about "Sharky", a tooth-brushing shark superhero. Make each sentence a bullet point.',
            ],
            'history_and_training_disabled': False,
            'conversation_mode': {
                'kind': 'primary_assistant',
            },
            'force_paragen': False,
            'force_paragen_model_slug': '',
            'force_nulligen': False,
            'force_rate_limit': False,
            'reset_rate_limits': False,
            'websocket_request_id': str(uuid.uuid4()),
            'system_hints': [],
            'force_use_sse': True,
            'conversation_origin': None,
            'client_contextual_info': {
                'is_dark_mode': True,
                'time_since_loaded': random.randint(22,33),
                'page_height': random.randint(600, 900),
                'page_width': random.randint(500, 800),
                'pixel_ratio': 2,
                'screen_height': random.randint(800, 1200),
                'screen_width': random.randint(1200, 2000),
            },
        }
        
        response = session.post('https://chatgpt.com/backend-anon/conversation',
            headers=headers, json=json_data, stream=True)
        
        replace = ''
        for line in response.iter_lines():
            if line:
                if 'DONE' in line.decode():
                    break
                
                data = json.loads(line.decode()[6:])
                if data.get('message').get('author').get('role') == 'assistant':
                    tokens = (data.get('message').get('content').get('parts')[0])
                    
                    yield tokens.replace(replace, '')
                    
                    replace = tokens


from __future__ import annotations

import os
import time
import random
import string
import logging
import asyncio
from typing import Union
from ..providers.base_provider import AsyncGeneratorProvider
from ..image import ImageResponse, to_image, to_data_uri
from ..typing import Union, Iterator, Messages, ImageType
from ..providers.types import BaseProvider, ProviderType, FinishReason
from ..providers.conversation import BaseConversation
from ..image import ImageResponse as ImageProviderResponse
from ..errors import NoImageResponseError
from .stubs import ChatCompletion, ChatCompletionChunk, Image, ImagesResponse
from .image_models import ImageModels
from .types import IterResponse, ImageProvider
from .types import Client as BaseClient
from .service import get_model_and_provider, get_last_provider
from .helper import find_stop, filter_json, filter_none
from ..models import ModelUtils
from ..Provider import IterListProvider


def iter_response(
    response: Iterator[str],
    stream: bool,
    response_format: dict = None,
    max_tokens: int = None,
    stop: list = None
) -> IterResponse:
    content = ""
    finish_reason = None
    completion_id = ''.join(random.choices(string.ascii_letters + string.digits, k=28))
    
    for idx, chunk in enumerate(response):
        if isinstance(chunk, FinishReason):
            finish_reason = chunk.reason
            break
        elif isinstance(chunk, BaseConversation):
            yield chunk
            continue
        
        content += str(chunk)
        
        if max_tokens is not None and idx + 1 >= max_tokens:
            finish_reason = "length"
        
        first, content, chunk = find_stop(stop, content, chunk if stream else None)
        
        if first != -1:
            finish_reason = "stop"
        
        if stream:
            yield ChatCompletionChunk(chunk, None, completion_id, int(time.time()))
        
        if finish_reason is not None:
            break
    
    finish_reason = "stop" if finish_reason is None else finish_reason
    
    if stream:
        yield ChatCompletionChunk(None, finish_reason, completion_id, int(time.time()))
    else:
        if response_format is not None and "type" in response_format:
            if response_format["type"] == "json_object":
                content = filter_json(content)
        yield ChatCompletion(content, finish_reason, completion_id, int(time.time()))


def iter_append_model_and_provider(response: IterResponse) -> IterResponse:
    last_provider = None
    for chunk in response:
        last_provider = get_last_provider(True) if last_provider is None else last_provider
        chunk.model = last_provider.get("model")
        chunk.provider = last_provider.get("name")
        yield chunk


class Client(BaseClient):
    def __init__(
        self,
        provider: ProviderType = None,
        image_provider: ImageProvider = None,
        **kwargs
    ) -> None:
        super().__init__(**kwargs)
        self.chat: Chat = Chat(self, provider)
        self._images: Images = Images(self, image_provider)

    @property
    def images(self) -> Images:
        return self._images

    async def async_images(self) -> Images:
        return self._images


class Completions:
    def __init__(self, client: Client, provider: ProviderType = None):
        self.client: Client = client
        self.provider: ProviderType = provider

    def create(
        self,
        messages: Messages,
        model: str,
        provider: ProviderType = None,
        stream: bool = False,
        proxy: str = None,
        response_format: dict = None,
        max_tokens: int = None,
        stop: Union[list[str], str] = None,
        api_key: str = None,
        ignored: list[str] = None,
        ignore_working: bool = False,
        ignore_stream: bool = False,
        **kwargs
    ) -> Union[ChatCompletion, Iterator[ChatCompletionChunk]]:
        model, provider = get_model_and_provider(
            model,
            self.provider if provider is None else provider,
            stream,
            ignored,
            ignore_working,
            ignore_stream,
        )

        stop = [stop] if isinstance(stop, str) else stop
        
        response = provider.create_completion(
            model,
            messages,
            stream=stream,
            **filter_none(
                proxy=self.client.get_proxy() if proxy is None else proxy,
                max_tokens=max_tokens,
                stop=stop,
                api_key=self.client.api_key if api_key is None else api_key
            ),
            **kwargs
        )
        
        response = iter_response(response, stream, response_format, max_tokens, stop)
        response = iter_append_model_and_provider(response)
        
        return response if stream else next(response)


class Chat:
    completions: Completions

    def __init__(self, client: Client, provider: ProviderType = None):
        self.completions = Completions(client, provider)


def iter_image_response(response: Iterator) -> Union[ImagesResponse, None]:
    logging.info("Starting iter_image_response")
    response_list = list(response)
    logging.info(f"Response list: {response_list}")
    
    for chunk in response_list:
        logging.info(f"Processing chunk: {chunk}")
        if isinstance(chunk, ImageProviderResponse):
            logging.info("Found ImageProviderResponse")
            return ImagesResponse([Image(image) for image in chunk.get_list()])
    
    logging.warning("No ImageProviderResponse found in the response")
    return None


def create_image(client: Client, provider: ProviderType, prompt: str, model: str = "", **kwargs) -> Iterator:
    logging.info(f"Creating image with provider: {provider}, model: {model}, prompt: {prompt}")
    
    if isinstance(provider, type) and provider.__name__ == "You":
        kwargs["chat_mode"] = "create"
    else:
        prompt = f"create an image with: {prompt}"
    
    response = provider.create_completion(
        model,
        [{"role": "user", "content": prompt}],
        stream=True,
        proxy=client.get_proxy(),
        **kwargs
    )
    
    logging.info(f"Response from create_completion: {response}")
    return response


class Images:
    def __init__(self, client: 'Client', provider: ImageProvider = None):
        self.client: 'Client' = client
        self.provider: ImageProvider = provider
        self.models: ImageModels = ImageModels(client)

    def generate(self, prompt: str, model: str = None, **kwargs) -> ImagesResponse:
        logging.info(f"Starting synchronous image generation for model: {model}, prompt: {prompt}")
        try:
            loop = asyncio.get_event_loop()
        except RuntimeError:
            loop = asyncio.new_event_loop()
            asyncio.set_event_loop(loop)
        
        try:
            result = loop.run_until_complete(self.async_generate(prompt, model, **kwargs))
            logging.info(f"Synchronous image generation completed. Result: {result}")
            return result
        except Exception as e:
            logging.error(f"Error in synchronous image generation: {str(e)}")
            raise
        finally:
            if loop.is_running():
                loop.close()

    async def async_generate(self, prompt: str, model: str = None, **kwargs) -> ImagesResponse:
        logging.info(f"Generating image for model: {model}, prompt: {prompt}")
        provider = self.models.get(model, self.provider)
        if provider is None:
            raise ValueError(f"Unknown model: {model}")
        
        logging.info(f"Provider: {provider}")
        
        if isinstance(provider, IterListProvider):
            if provider.providers:
                provider = provider.providers[0]
                logging.info(f"Using first provider from IterListProvider: {provider}")
            else:
                raise ValueError(f"IterListProvider for model {model} has no providers")

        if isinstance(provider, type) and issubclass(provider, AsyncGeneratorProvider):
            logging.info("Using AsyncGeneratorProvider")
            messages = [{"role": "user", "content": prompt}]
            async for response in provider.create_async_generator(model, messages, **kwargs):
                if isinstance(response, ImageResponse):
                    return self._process_image_response(response)
                elif isinstance(response, str):
                    image_response = ImageResponse([response], prompt)
                    return self._process_image_response(image_response)
        elif hasattr(provider, 'create'):
            logging.info("Using provider's create method")
            if asyncio.iscoroutinefunction(provider.create):
                response = await provider.create(prompt)
            else:
                response = provider.create(prompt)
            
            if isinstance(response, ImageResponse):
                return self._process_image_response(response)
            elif isinstance(response, str):
                image_response = ImageResponse([response], prompt)
                return self._process_image_response(image_response)
        else:
            raise ValueError(f"Provider {provider} does not support image generation")
        
        logging.error(f"Unexpected response type: {type(response)}")
        raise NoImageResponseError(f"Unexpected response type: {type(response)}")

    def _process_image_response(self, response: ImageResponse) -> ImagesResponse:
        processed_images = []
        for image_data in response.get_list():
            if image_data.startswith('http://') or image_data.startswith('https://'):
                processed_images.append(Image(url=image_data))
            else:
                image = to_image(image_data)
                file_name = self._save_image(image)
                processed_images.append(Image(url=file_name))
        return ImagesResponse(processed_images)

    def _save_image(self, image: 'PILImage') -> str:
        os.makedirs('generated_images', exist_ok=True)
        file_name = f"generated_images/image_{int(time.time())}.png"
        image.save(file_name)
        return file_name

    async def create_variation(self, image: Union[str, bytes], model: str = None, **kwargs):
        provider = self.models.get(model, self.provider)
        if provider is None:
            raise ValueError(f"Unknown model: {model}")

        if isinstance(provider, type) and issubclass(provider, AsyncGeneratorProvider):
            messages = [{"role": "user", "content": "create a variation of this image"}]
            image_data = to_data_uri(image)
            async for response in provider.create_async_generator(model, messages, image=image_data, **kwargs):
                if isinstance(response, ImageResponse):
                    return self._process_image_response(response)
                elif isinstance(response, str):
                    image_response = ImageResponse([response], "Image variation")
                    return self._process_image_response(image_response)
        elif hasattr(provider, 'create_variation'):
            if asyncio.iscoroutinefunction(provider.create_variation):
                response = await provider.create_variation(image, **kwargs)
            else:
                response = provider.create_variation(image, **kwargs)
            
            if isinstance(response, ImageResponse):
                return self._process_image_response(response)
            elif isinstance(response, str):
                image_response = ImageResponse([response], "Image variation")
                return self._process_image_response(image_response)
        else:
            raise ValueError(f"Provider {provider} does not support image variation")

        raise NoImageResponseError("Failed to create image variation")




from __future__ import annotations
import json
from aiohttp import ClientSession

from ..typing import AsyncResult, Messages
from .base_provider import AsyncGeneratorProvider, ProviderModelMixin
from .helper import format_prompt
from ..image import ImageResponse

class Nexra(AsyncGeneratorProvider, ProviderModelMixin):
    url = "https://nexra.aryahcr.cc"
    chat_api_endpoint = "https://nexra.aryahcr.cc/api/chat/gpt"
    image_api_endpoint = "https://nexra.aryahcr.cc/api/image/complements"
    working = True
    supports_gpt_35_turbo = True
    supports_gpt_4 = True
    supports_system_message = True
    supports_message_history = True
    
    default_model = 'gpt-3.5-turbo'
    text_models = [
        'gpt-4', 'gpt-4-0613', 'gpt-4-32k', 'gpt-4-0314', 'gpt-4-32k-0314',
        'gpt-3.5-turbo', 'gpt-3.5-turbo-16k', 'gpt-3.5-turbo-0613', 'gpt-3.5-turbo-16k-0613', 'gpt-3.5-turbo-0301',
        'gpt-3', 'text-davinci-003', 'text-davinci-002', 'code-davinci-002',
        'text-curie-001', 'text-babbage-001', 'text-ada-001',
        'davinci', 'curie', 'babbage', 'ada', 'babbage-002', 'davinci-002',
    ]
    image_models = ['dalle', 'dalle2', 'dalle-mini', 'emi']
    models = [*text_models, *image_models]
    
    model_aliases = {
        "gpt-4": "gpt-4-0613",
        "gpt-4": "gpt-4-32k",
        "gpt-4": "gpt-4-0314",
        "gpt-4": "gpt-4-32k-0314",
        
        "gpt-3.5-turbo": "gpt-3.5-turbo-16k",
        "gpt-3.5-turbo": "gpt-3.5-turbo-0613",
        "gpt-3.5-turbo": "gpt-3.5-turbo-16k-0613",
        "gpt-3.5-turbo": "gpt-3.5-turbo-0301",
        
        "gpt-3": "text-davinci-003",
        "gpt-3": "text-davinci-002",
        "gpt-3": "code-davinci-002",
        "gpt-3": "text-curie-001",
        "gpt-3": "text-babbage-001",
        "gpt-3": "text-ada-001",
        "gpt-3": "text-ada-001",
        "gpt-3": "davinci",
        "gpt-3": "curie",
        "gpt-3": "babbage",
        "gpt-3": "ada",
        "gpt-3": "babbage-002",
        "gpt-3": "davinci-002",
        
        "dalle-2": "dalle2",
    }
    
    @classmethod
    def get_model(cls, model: str) -> str:
        if model in cls.text_models or model in cls.image_models:
            return model
        elif model in cls.model_aliases:
            return cls.model_aliases[model]
        else:
            return cls.default_model

    @classmethod
    async def create_async_generator(
        cls,
        model: str,
        messages: Messages,
        proxy: str = None,
        **kwargs
    ) -> AsyncResult:
        model = cls.get_model(model)
        
        headers = {
            "Content-Type": "application/json",
        }
        
        async with ClientSession(headers=headers) as session:
            if model in cls.image_models:
                # Image generation
                prompt = messages[-1]['content'] if messages else ""
                data = {
                    "prompt": prompt,
                    "model": model,
                    "response": "url"
                }
                async with session.post(cls.image_api_endpoint, json=data, proxy=proxy) as response:
                    response.raise_for_status()
                    result = await response.text()
                    result_json = json.loads(result.strip('_'))
                    image_url = result_json['images'][0] if result_json['images'] else None
                    
                    if image_url:
                        yield ImageResponse(images=image_url, alt=prompt)
            else:
                # Text completion
                data = {
                    "messages": messages,
                    "prompt": format_prompt(messages),
                    "model": model,
                    "markdown": False
                }
                async with session.post(cls.chat_api_endpoint, json=data, proxy=proxy) as response:
                    response.raise_for_status()
                    result = await response.text()
                    
                    try:
                        json_response = json.loads(result)
                        gpt_response = json_response.get('gpt', '')
                        yield gpt_response
                    except json.JSONDecodeError:
                        yield result



from __future__ import annotations

import json
import re
from aiohttp import ClientSession

from ..typing import AsyncResult, Messages
from .base_provider import AsyncGeneratorProvider, ProviderModelMixin
from .helper import format_prompt

class TwitterBio(AsyncGeneratorProvider, ProviderModelMixin):
    url = "https://www.twitterbio.io"
    api_endpoint_mistral = "https://www.twitterbio.io/api/mistral"
    api_endpoint_openai = "https://www.twitterbio.io/api/openai"
    working = True
    supports_gpt_35_turbo = True
    
    default_model = 'gpt-3.5-turbo'
    models = [
        'mistralai/Mixtral-8x7B-Instruct-v0.1',
        'gpt-3.5-turbo',
    ]
    
    model_aliases = {
        "mixtral-8x7b": "mistralai/Mixtral-8x7B-Instruct-v0.1",
    }

    @classmethod
    def get_model(cls, model: str) -> str:
        if model in cls.models:
            return model
        return cls.default_model

    @staticmethod
    def format_text(text: str) -> str:
        text = re.sub(r'\s+', ' ', text.strip())
        text = re.sub(r'\s+([,.!?])', r'\1', text)
        return text

    @classmethod
    async def create_async_generator(
        cls,
        model: str,
        messages: Messages,
        proxy: str = None,
        **kwargs
    ) -> AsyncResult:
        model = cls.get_model(model)
        
        headers = {
            "accept": "*/*",
            "accept-language": "en-US,en;q=0.9",
            "cache-control": "no-cache",
            "content-type": "application/json",
            "origin": cls.url,
            "pragma": "no-cache",
            "priority": "u=1, i",
            "referer": f"{cls.url}/",
            "sec-ch-ua": '"Chromium";v="127", "Not)A;Brand";v="99"',
            "sec-ch-ua-mobile": "?0",
            "sec-ch-ua-platform": '"Linux"',
            "sec-fetch-dest": "empty",
            "sec-fetch-mode": "cors",
            "sec-fetch-site": "same-origin",
            "user-agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/127.0.0.0 Safari/537.36"
        }
        async with ClientSession(headers=headers) as session:
            prompt = format_prompt(messages)
            data = {
                "prompt": f'{prompt}.'
            }
            
            if model == 'mistralai/Mixtral-8x7B-Instruct-v0.1':
                api_endpoint = cls.api_endpoint_mistral
            elif model == 'gpt-3.5-turbo':
                api_endpoint = cls.api_endpoint_openai
            else:
                raise ValueError(f"Unsupported model: {model}")
            
            async with session.post(api_endpoint, json=data, proxy=proxy) as response:
                response.raise_for_status()
                buffer = ""
                async for line in response.content:
                    line = line.decode('utf-8').strip()
                    if line.startswith('data: '):
                        try:
                            json_data = json.loads(line[6:])
                            if model == 'mistralai/Mixtral-8x7B-Instruct-v0.1':
                                if 'choices' in json_data and len(json_data['choices']) > 0:
                                    text = json_data['choices'][0].get('text', '')
                                    if text:
                                        buffer += text
                            elif model == 'gpt-3.5-turbo':
                                text = json_data.get('text', '')
                                if text:
                                    buffer += text
                        except json.JSONDecodeError:
                            continue
                    elif line == 'data: [DONE]':
                        break
                
                if buffer:
                    yield cls.format_text(buffer)
